local addon, DRTS = ...

Treasures_TreasureDataProviderMixin = CreateFromMixins(MapCanvasDataProviderMixin);

function Treasures_TreasureDataProviderMixin:OnAdded(mapCanvas)
    MapCanvasDataProviderMixin.OnAdded(self, mapCanvas);
end

function Treasures_TreasureDataProviderMixin:OnRemoved(mapCanvas)
    MapCanvasDataProviderMixin.OnRemoved(self, mapCanvas);
end

function Treasures_TreasureDataProviderMixin:OnShow()
    print("Treasures_TreasureDataProviderMixin:OnShow");
    self:RefreshAllData();
end

function Treasures_TreasureDataProviderMixin:OnHide()
    print("Treasures_TreasureDataProviderMixin:OnHide");
    self:RemoveAllData();
end

function Treasures_TreasureDataProviderMixin:RemoveAllData()
    self:GetMap():RemoveAllPinsByTemplate("Treasures_HighlightableTreasurePinTemplate");
end

function Treasures_TreasureDataProviderMixin:RefreshAllData(fromOnShow)
    self:RemoveAllData();

    local mapID = self:GetMap():GetMapID();

    --print("Treasures_TreasureDataProviderMixin:RefreshAllData", mapID);
    self:RefreshTreasures(mapID);
end

local TreasureContinentsIds = {
    [1116] = 13, -- Draenor
    [1152] = 13, -- Garrison Level 1 (H)
    [1330] = 13, -- Garrison Level 2 (H)
    [1153] = 13, -- Garrison Level 3 (H)
    [1158] = 13, -- Garrison Level 1 (A)
    [1331] = 13, -- Garrison Level 2 (A)
    [1159] = 13, -- Garrison Level 3 (A)
    [1464] = 13, -- Tanaan Jungle
    [1220] = 14, -- Broken Isles
}

local TreasureQuestsDesc = {
    [36466] = "Talador",
    [36467] = "Spires of Arak",
    [36465] = "Gorgrond",
    [34557] = "Frostfire",
    [36468] = "Nagrand",
    [36464] = "Shadowmoon Valley",
    [36249] = "requires Lumber Yard",
    [36250] = "requires Lumber Yard",
    [36251] = "requires Sparring Arena",
    [36252] = "requires Sparring Arena",
    [39463] = "Tanaan Jungle",
}

local TreasuresContinentMap = {
    [13] = 1116,
    [14] = 1220,
}

local function GetDistanceToPoint(pointx, pointy, x, y)
    local dX = pointx - x
    local dY = pointy - y
    return (dX * dX + dY * dY) ^ 0.5
end

local function SelectQuest(treasure, type)
    if treasure.flags == 131073 then
        if type == "map" then
            return treasure.quests[1]
        elseif type == "complete" then
            return treasure.quests[2]
        else
            --print("Unknown type "..type.. " for flags 131073")
            return 0
        end
    elseif treasure.flags == 65541 then
        if type == "map" then
            return treasure.quests[2]
        elseif type == "complete" then
            return treasure.quests[1]
        else
            --print("Unknown type "..type.. " for flags 65541")
            return 0
        end
    elseif treasure.flags == 65537 then
        if type == "map" then
            return treasure.quests[2]
        elseif type == "complete" then
            return treasure.quests[1]
        else
            --print("Unknown type "..type.. " for flags 65537")
            return 0
        end
    elseif treasure.flags == 524310 then
        if type == "choise1" then
            return treasure.quests[1]
        elseif type == "choise2" then
            return treasure.quests[2]
        elseif type == "map" then
            return treasure.quests[3]
        elseif type == "complete" then
            return treasure.quests[4]
        else
            --print("Unknown type "..type.. " for flags 524310")
            return 0
        end
    elseif treasure.flags == 0 then
        --print("Treasure "..treasure.id.." has 0 flags")
        return 0
    else
        --print("Unknown flags "..treasure.flags)
        return 0
    end
end

local function GetTreasureCompletion(treasure)
    local trquest, tmquest, c1quest, c2quest = SelectQuest(treasure, "complete"), SelectQuest(treasure, "map"), SelectQuest(treasure, "choise1"), SelectQuest(treasure, "choise2")
    local trCheck = trquest == 0 or not IsQuestFlaggedCompleted(trquest)
    local tmCheck = DRTSOptions.ignoretm or tmquest == 0 or not IsQuestFlaggedCompleted(tmquest)
    local choise1Check = c1quest == 0 or IsQuestFlaggedCompleted(c1quest)
    local choise2Check = c2quest == 0 or IsQuestFlaggedCompleted(c2quest)
    return trCheck, tmCheck, choise1Check, choise2Check, trquest, tmquest, c1quest, c2quest
end

local function GetTreasureById(id)
	for _,v in pairs(DRTS.treasures) do
		if v.id == id then
			return v;
		end
	end
end

--LoadAddOn("Blizzard_DebugTools")

function Treasures_TreasureDataProviderMixin:RefreshTreasures(mapID)
    print("Treasures_TreasureDataProviderMixin:RefreshTreasures", mapID);

    local uiMapInfo = C_Map.GetMapInfo(mapID);
    --DevTools_Dump(uiMapInfo);

    local continentID2 = C_Map.GetWorldPosFromMapPos(mapID, CreateVector2D(0.5, 0.5));

    local isContinent = uiMapInfo.mapType == Enum.UIMapType.World or uiMapInfo.mapType == Enum.UIMapType.Continent;

    local showTreasures = (not isContinent) and DRTSOptions.enabled;

    for _,v in pairs(DRTS.treasures) do
        local continentID = TreasuresContinentMap[v.continent];
        if continentID and continentID2 and continentID == continentID2 then
            local uiMapID, mapPosition = C_Map.GetMapPosFromWorldPos(continentID, CreateVector2D(v.x, v.y), mapID);

            local trCheck, tmCheck, choise1Check, choise2Check, _, tmquest, c1quest, _ = GetTreasureCompletion(v);

            if showTreasures and uiMapID and mapPosition and uiMapID == mapID and (DRTSOptions.debug or (not DRTSOptions.hidden[v.id] and trCheck and tmCheck and choise1Check and choise2Check)) then
                --print(v.name);

                local x, y = mapPosition:GetXY();

                local name;
                if DRTSOptions.debug then
                    name = format("%s (%u, %.02f, %.02f)", v.name, v.id, x * 100, y * 100)
                else
                    name = format("%s (%.02f, %.02f)", v.name, x * 100, y * 100)
                end

                local description = v.desc ~= "" and v.desc or format("Treasure: %s!", TreasureQuestsDesc[c1quest] or TreasureQuestsDesc[tmquest] or "Unknown");

                local poiInfo = {
                    id = v.id,
                    name = name,
                    desc = description,
                    position = mapPosition
                };
                local pin = self:GetMap():AcquirePin("Treasures_HighlightableTreasurePinTemplate", poiInfo);
                pin:Show();
            end            
        end
    end
end

Treasures_TreasurePinMixin = CreateFromMixins(MapCanvasPinMixin);

function Treasures_TreasurePinMixin:OnLoad()
    self:SetScalingLimits(1, 1.0, 1.2);
    self:UseFrameLevelType("PIN_FRAME_LEVEL_AREA_POI");
end

function Treasures_TreasurePinMixin:OnAcquired(poiInfo)
    --print("Treasures_TreasurePinMixin:OnAcquired");
    self:SetTexture();

    self.id = poiInfo.id;
    self.name = poiInfo.name;
    self.description = poiInfo.desc;

    self:SetPosition(poiInfo.position:GetXY());
end

function Treasures_TreasurePinMixin:OnMouseEnter(motion)
    --print("Treasures_TreasurePinMixin:OnMouseEnter");
    local name, description;
    for pin in self:GetMap():EnumeratePinsByTemplate("Treasures_HighlightableTreasurePinTemplate") do
        if pin:IsMouseOver() then
            name = pin.name;
            description = pin.description;
        end
    end
    if name then
        WorldMapTooltip:SetOwner(self);
        WorldMapTooltip:SetText(HIGHLIGHT_FONT_COLOR:WrapTextInColorCode(name));
        if description then
        	WorldMapTooltip:AddLine(NORMAL_FONT_COLOR:WrapTextInColorCode(description));
        end
        WorldMapTooltip:Show();
    end
end

function Treasures_TreasurePinMixin:OnMouseLeave(motion)
    --print("Treasures_TreasurePinMixin:OnMouseLeave");
    WorldMapTooltip:Hide();
end

function Treasures_TreasurePinMixin:OnMouseDown(button)
    WorldMapFrame.ScrollContainer:OnMouseDown(button);
end

local debugMsgFormat = "Treasure %s (%u):\nxpos %f\nypos %f\ntracking quest %u (%s)\ntreasuremap quest %u (%s)\ndistance %.02f"

function Treasures_TreasurePinMixin:OnMouseUp(button)
    --print("Treasures_TreasurePinMixin:OnMouseUp", button);
    if not DRTSOptions.debug or button == "RightButton" then
        WorldMapFrame.ScrollContainer:OnMouseUp(button);
        return;
    end
    local t = GetTreasureById(self.id);
    local trCheck, tmCheck, _, _, trquest, tmquest, _, _ = GetTreasureCompletion(t);
    local trstate = trquest ~= 0 and (trCheck and "Not completed" or "Completed") or "No quest";
    local tmstate = tmquest ~= 0 and (tmCheck and "Not completed" or "Completed") or "No quest";
    local x, y = UnitPosition("player");
    print(format(debugMsgFormat, t.name, t.id, t.x, t.y, trquest, trstate, tmquest, tmstate, GetDistanceToPoint(x, y, t.x, t.y)));
end

function Treasures_TreasurePinMixin:SetTexture()
    self:SetSize(24, 24);
    self.Texture:SetWidth(12);
    self.Texture:SetHeight(12);
    self.Texture:SetTexture("Interface/Minimap/POIIcons");
    if self.HighlightTexture then
        self.HighlightTexture:SetTexture("Interface/Minimap/POIIcons");
    end

    local x1, x2, y1, y2 = GetPOITextureCoords(197);
    self.Texture:SetTexCoord(x1, x2, y1, y2);
    if self.HighlightTexture then
        self.HighlightTexture:SetTexCoord(x1, x2, y1, y2);
    end
end

-- /dump C_TaskQuest.GetQuestLocation(36603, 543)
